import createApp from '../../util/express-app';
import createAuthenticatedHandler from '../../util/create-authenticated-handler';
import { RequestContext } from '../../util/request-context.type';
import { Response } from 'express';
import { getErrorMessage } from '../../util/error-message';
import { getChartsByMd5, getChartsBySha256 } from './chart.service';
import { flatten, keyBy } from 'lodash-es';

const app = createApp();
export const handler = createAuthenticatedHandler(app);

app.post('/chart/public', [
  async (req: RequestContext, res: Response) => {
    try {
      const { md5 = [], sha256 = [] } = req.body as {
        md5: string[];
        sha256: string[];
      };

      /** @TODO Ask herman how to batch query */
      const md5Charts = flatten(
        await Promise.all(md5.map((md5) => getChartsByMd5(md5))),
      );
      const md5ChartsKeyedByMd5 = keyBy(md5Charts, 'md5');

      console.log(md5Charts);

      const sha256Charts = flatten(
        await Promise.all(sha256.map((sha256) => getChartsBySha256(sha256))),
      );
      const sha256ChartsKeyedBySha256 = keyBy(sha256Charts, 'sha256');

      res.json({
        md5: md5ChartsKeyedByMd5,
        sha256: sha256ChartsKeyedBySha256,
      });
    } catch (error) {
      const message = getErrorMessage(error);
      console.error(`Failed to get charts ${message}`);
      res.status(400).json({ error: 'Failed to get charts' });
    }
  },
]);
